@@ -1,5 +1,7 @@
 
-auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,beta_eta,
+
+
+auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
                          maxit.mean,eta,u_h,v_h,Sig,
                          control.HLfit,
                          X.pv,
@@ -11,6 +13,8 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
                          corrPars, ## only for error message in intervalStep
                          processed
 ) {
+  if (inherits(eta,"Matrix")) eta <- as.matrix(eta) ## eh oui...
+  ### 
   BinomialDen <- processed$BinomialDen
   conv.threshold <- processed$conv.threshold
   stop.on.error <- processed$stop.on.error
@@ -27,10 +31,10 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
   pforpv <- ncol(X.pv)
   nobs <- NROW(X.pv)
   betaV <- c(beta_eta,v_h)   
-  
-  ## les deux arguments sont apparus lors d'une etape de restructuration du code: 
-  ##ils deaient venir d'une version locale de auglinmodfit dans HLfit
-  ## les autres variables venant directement de la frame HLfit
+  ## t(ZAL) used repeatedly, for constant ZAL, within this function 
+  #if ( ! inherits(ZAL,"identityMatrix")) {
+    tZAL <- as.matrix(t(ZAL))
+  #} else tZAL <- ZAL
   `compute.sscaled` <- function(sqrt.ww,qr.d2hdv2) { ## needs qr.d2hdv2, ZAL, stop.on.error, d2hdv2, rWW, ZALI, family, dmudeta, BinomialDen, mu, eta... ETC
     ########## HL(1,.) adjustment for mean ################## and specifically the a(1) term in LeeL 12 p. 963
     ## if LMM (ie resp gaussian, ranef gaussian), all coef<x> are 0 -> correction is 0 (but this fn must not have been called)
@@ -85,23 +89,7 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
     if (any(is.na(c(vecdi1,vecdi2,vecdi3)))) { ## then we need to compute some of them
       ## P is P in LeeL appendix p. 4 and is P_R in MolasL p. 3307 
       ## looks like leverage computation, but the ZALI columns are considered instead of the full augmented design matrix 
-      wAugZALI <- sweep(attr(ZAL,"ZALI"),MARGIN=1,sqrt.ww,`*`) # rWW %*% ZALI ## rWW previously computed for leverage computation
-      ## next two lines will be slow for very large matrices but the leverages() function using RcppEigen is even slower
-      partition <- attr(ZAL,"partition")
-      if ( !is.null(partition) ) { ## use block structure in ZAL;  
-        nrI <- nrow(I)
-        Pdiag <- rep(0,nrow(wAugZALI))
-        abyss <- sapply(seq_len(length(partition)-1),function(v) {
-          sequ <- (partition[v]+1):partition[v+1] 
-          colonnes <- wAugZALI[,sequ,drop=F]
-          qrcolonnes <- qr(colonnes)
-          levs <- rowSums(qr.qy(qrcolonnes, diag(1, nrow = nrow(colonnes), ncol = ncol(colonnes)))[c(sequ,sequ+nrI),,drop=FALSE]^2)
-          Pdiag[c(sequ,sequ+nrI)] <<- levs ## proceeds by this side-effect
-        })
-      } else { ## straightforward but does not use block structure
-        qrwAugZALI <- qr(wAugZALI)
-        Pdiag <- rowSums(qr.qy(qrwAugZALI, diag(1, nrow = nrow(wAugZALI), ncol = ncol(wAugZALI)))^2)
-      }
+      Pdiag <- calc.Pdiag(ZAL=ZAL,sqrt.ww=sqrt.ww)
       #
       nrand <- length(lcrandfamfam)
       seqn_u_h <- seq_len(cum_n_u_h[nrand+1L])
@@ -109,15 +97,15 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
       if (is.na(vecdi1)) vecdi1 <- Pdiagn * coef1
       # K2 is K2 matrix in LeeL appendix p. 4 and is -D in MolasL p. 3307 
       # W is Sigma^-1 ; TWT = t(ZALI)%*%W%*%ZALI = ZAL'.Wresid.ZAL+Wranef = -d2hdv2 !
-      K2 <- solveWrap.matrix(qr.d2hdv2,t(ZAL),stop.on.error=stop.on.error) ## t(ZAL) missing in some implementations... no effect with Gaussian ranefs...  
+      K2 <- solveWrap.matrix(qr.d2hdv2,tZAL,stop.on.error=stop.on.error) ## t(ZAL) missing in some implementations... no effect with Gaussian ranefs...  
       if (class(K2)=="try-error") {
         mess <- pastefrom("problem in 'K2' computation.",prefix="(!) From ") ## cf BB 877
         warning(mess)
-        K2 <- ginv(d2hdv2) %*% t(ZAL)            
+        K2 <- ginv(d2hdv2) %*% tZAL            
       } ## so far we have computed (d2hdv2)^{-1}.t(Z)= -(TWT)^{-1}.t(Z)
       if (is.na(vecdi2)) {
         # K1 is K1 in LeeL appendix p. 4 and is A=-ZD in MolasL p. 3307-8 
-        K1 <- ZAL %*% K2
+        K1 <- ZAL %id*id% K2
         vecdi2<-rep(0,nobs) 
         for (i in 1:nobs) vecdi2[i] <- sum( Pdiagn * coef2 * K1[1:nobs,i] )                 
       }
@@ -160,16 +148,15 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
     sqrt.w1 <- sqrt(w.resid) ## if maxit.mean>1 GLM weights have been changed and the following must be recomputed
     sqrt.w2 <- sqrt(w.ranef) ##         
     sqrt.ww <- c(sqrt.w1,sqrt.w2) 
-    wAugX <- sweepZ1W(TT,sqrt.ww) # sweep(TT,MARGIN=1,sqrt.ww,`*`) # rWW %*%TT
-    if (  !  .spaMM.data$options$USEEIGEN)  {
-      if ( .spaMM.data$options$USE_SPARSE_Q && ( ! inherits(wAugX,"Matrix"))  ) wAugX <- Matrix(wAugX) ## cf ?Matrix for default wrt sparsity 
-      SQR <- QRwrap(wAugX)
-    } 
+    if (inherits(TT,"Matrix")) {
+      wAugX <- Diagonal(x=sqrt.ww) %*% TT ## apparemment sweep vriament pas efficace sur Matrix
+    } else wAugX <- sweepZ1W(TT,sqrt.ww) # sweep(TT,MARGIN=1,sqrt.ww,`*`) # rWW %*%TT
     old_betaV <- betaV
     ######## According to 'theorem 1' of LeeL12
     ## new beta estimate from z1-a(i)
     ## where z1 is
     z1 <- eta+(y-mu)/dmudeta-off ## LeeNP 182 bas. GLM-adjusted response variable; O(n)*O(1/n)
+    if (inherits(z1,"Matrix")) z1 <- as.numeric(z1) ## conversion from 1-col dgCMatrix because c(z1 dgCMatrix,z2 numeric) does not work
     ## and a(i) (for HL(i,1)) is a(0) or a(0)+ something
     ## and a(0) depends on z2, as follows :
     if (all(lcrandfamfam=="gaussian")) {
@@ -189,11 +176,11 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
       #        nXn  .   nXn      nX'r'    'r'X'r'       'r'X'r'    'r'
       # a <- Sig %*% Wresid %*% ZAL %*% solve(-d2hdv2) %*% Wranef %*% z2 ## p. 963 l. 1-2; a(0) supp mat p. 6 
       aa <- w.ranef * z2
-      if (is.null(qr.d2hdv2)) { 
+      if (is.null(attr(d2hdv2,"qr"))) { 
         a <- try(solve(d2hdv2, - aa),silent=TRUE)
         if (class(a)=="try-error") {
-          qr.d2hdv2 <- QRwrap(d2hdv2) 
-          a <- solveWrap.vector(qr.d2hdv2,  -aa,stop.on.error=stop.on.error)
+          attr(d2hdv2,"qr") <- QRwrap(d2hdv2)
+          a <- solveWrap.vector(attr(d2hdv2,"qr"),  -aa,stop.on.error=stop.on.error)
           ## FR->FR patch: 
           if (class(a)=="try-error") {
             mess <- pastefrom("the Hessian matrix appears singular. Extreme lambda/phi value and/or extremely correlated random effects?",
@@ -209,20 +196,20 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
           }
         }  
       } else { ## we already have a qr, we use it
-        a <- solveWrap.vector(qr.d2hdv2, -aa,stop.on.error=stop.on.error)
+        a <- solveWrap.vector(attr(d2hdv2,"qr"), -aa,stop.on.error=stop.on.error)
       }    
-      a <- Sig %*% ( w.resid * (ZAL %*% a) ) ## a(0) in LeeL12
+      a <- Sig %*% ( w.resid * (ZAL %id*id% a) ) ## a(0) in LeeL12
       a <- as.numeric(a) ## incase it became a Matrix, which oddly does not fit with z1-a below...
     }         
     ## and the 'something' for a(1) is computed as follows
     if (HL[1]>0 && (! LMMbool )) { #pforpv>0 && removed since sscaled used for u_h estimation too...
-      if (is.null(qr.d2hdv2)) qr.d2hdv2 <- QRwrap(d2hdv2) 
-      sscaled <- compute.sscaled(sqrt.ww=sqrt.ww,qr.d2hdv2=qr.d2hdv2) ## s detadmu
+      if (is.null(attr(d2hdv2,"qr"))) {attr(d2hdv2,"qr") <- QRwrap(d2hdv2)} ## Cholwrap tested
+      sscaled <- compute.sscaled(sqrt.ww=sqrt.ww,qr.d2hdv2=attr(d2hdv2,"qr")) ## s detadmu
     } else sscaled <- 0L 
     ######## new estimates (tentative if LevenbergMM) 
     if (HL[1]=="SEM") {
       tXinvS <- NULL
-      v_h <- solve(-d2hdv2) %*% (t(ZAL)%*% ((z1 - X.pv %*% beta_eta) * w.resid)+ z2*w.ranef)          
+      v_h <- solve(-d2hdv2, (tZAL %*% ((z1 - X.pv %*% beta_eta) * w.resid)+ z2*w.ranef))          
       betaV <- c(beta_eta,v_h)
       dbetaV <- betaV - old_betaV
     } else if (betaFirst)  { ### LeeL12 top p. 6 Appendix (code non optimise, useful for checking other algorithms) 
@@ -233,7 +220,7 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
       rhs <-  tXinvS %*% (z1-a) ## already correct in 1.0
       qr.XtinvSX <- QRwrap(tXinvS%*%X.pv) ## looks contrived but avoids computing sqrt(Sig) (! not diag !); and XinvS%*%X.pv is small
       beta_eta <- solveWrap.vector( qr.XtinvSX , rhs ,stop.on.error=stop.on.error) 
-      v_h <- solve(-d2hdv2) %*% (t(ZAL)%*% ((z1 - X.pv %*% beta_eta) * w.resid)+ z2*w.ranef)  ## FR->FR pbb nonoptimal solve here        
+      v_h <- solve(-d2hdv2, (tZAL %*% ((z1 - X.pv %*% beta_eta) * w.resid)+ z2*w.ranef))        
       betaV <- c(beta_eta,v_h)
       dbetaV <- betaV - old_betaV
     } else { ### true augmented model, whether LevenbergM or not;
@@ -247,17 +234,17 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
         ## (2) the first operation in LevenbergMstep is to substract (eta^0,v^0): LM_wAugz <- wAugz - wAugX %*% betaV
         ##    so we keep (eta^0,v^0) here
         Augz <- c(z1- sscaled,
-                  z2+ ((1/w.ranef) * t(ZAL)) %*% (sscaled * w.resid ))  ## 
+                  z2+ ((1/w.ranef) * tZAL) %*% (sscaled * w.resid ))  ## 
         ## z2 correction  constructed from eqs (3)(4) of LeeL12 App. p5 and consistent with eq B1 MolasL:
         ## so that row 2 of wAugX.(z1-sscaled,z2+...) = Z'W1 z1 + W2 z2 => Z W1 sscaled - W2 (...) =0 => (...)=
       }
       wAugz <- Augz*sqrt.ww  
       
-      if ( maxit.mean > 1 && LevenbergM) { ## default LevenbergM
-        LM_wAugz <- wAugz - wAugX %*% betaV
+      if ( maxit.mean > 1 && LevenbergM) { ## default(for *G*LMM) LevenbergM
+        LM_wAugz <- wAugz - as.matrix(wAugX %*% betaV)
         ## verif correct rhs: verif_dbetaV <- safesolve.qr.vector(SQR, LM_wAugz,stop.on.error=stop.on.error)            
         if (.spaMM.data$options$USEEIGEN) {
-          bla <- LevenbergMstepCallingCpp(wAugX=wAugX,LM_wAugz=LM_wAugz,damping=damping)
+          bla <- LevenbergMstepCallingCpp(wAugX=as.matrix(wAugX),LM_wAugz=LM_wAugz,damping=damping)
         } else bla <- LevenbergMstep(wAugX=wAugX,LM_wAugz=LM_wAugz,damping=damping,stop.on.error=stop.on.error)
         dbetaV <- bla$dbetaV
         betaV <- betaV + dbetaV
@@ -272,14 +259,56 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
             intervalBlob <- intervalStep(old_betaV=old_betaV,wAugX=wAugX,wAugz=wAugz,
                                          currentp_v=currentp_v,
                                          intervalInfo=control.HLfit$intervalInfo,corrPars=corrPars)
-            betaVQ <- intervalBlob$betaVQ ; betaV <- intervalBlob$betaV
-          } else {
-            betaVQ <- lmwithQ(wAugX,wAugz)
-            betaV <- betaVQ$coef   
+            levQ <- intervalBlob$levQ ; betaV <- intervalBlob$betaV
+          } else { ## default for LMMs
+            if (inherits(wAugX,"Matrix")) {
+              ## DEFAULT COMES LAST
+              if (.spaMM.data$options$processedQRmethod == "Matrix::qr") {
+                  #xpx <- crossprod(wAugX)
+                #xpy <- crossprod(wAugX, wAugz)
+                #betaV <- solve(xpx,xpy) 
+                #levQ <- (wAugX %*% (chol2inv(chol(xpx)))%*% t(wAugX)) ## suremet améliorable
+                ## see http://cran.r-project.org/web/packages/Matrix/vignettes/Comparisons.pdf
+                qrwAugX <- Matrix::qr(wAugX)
+                betaV <- Matrix::qr.coef(qrwAugX,wAugz)
+                levQ <- as.matrix(Matrix::qr.Q(qrwAugX))
+              } else if (.spaMM.data$options$processedQRmethod == "lmwithSparseQ") {
+                message("lmwithSparseQ called -- should be devel code only") ## protection...
+                betaVQ <- lmwithSparseQ(wAugX,wAugz) ## tragically slow,; cf system.time's in commented code below
+                betaV <- betaVQ$coef
+                pivI <- sort.list(betaVQ$P) ## no pivoting with lmwithQ, pivoting with sparse
+                levQ <- as.matrix(betaVQ$Q_ap[,pivI][,seq_len(ncol(wAugX))]) # using Q_ap, not Q
+                #print(system.time(Rcpp_sparseQR(wAugX)))
+                #print(system.time(Rcpp_QR(as.matrix(wAugX))))
+                if (FALSE) { ## confinement de code de debugage 
+                  qrwAugX <- Matrix::qr(wAugX)
+                  essai <- Matrix::qr.Q(qrwAugX)
+                  print(max(abs(diag(tcrossprod(essai))-diag(tcrossprod(levQ)))))
+                  essai <- Matrix::qr.coef(qrwAugX,wAugz)
+                  print(max(abs(essai-betaV)))
+                }
+              } else if (.spaMM.data$options$processedQRmethod == "lmwithQ_sparseZAL") {     
+                betaVQ <- lmwithQ(as.matrix(wAugX),wAugz) ## slow step
+                betaV <- betaVQ$coef
+                levQ <- betaVQ$Q[,seq_len(ncol(wAugX))] ## Eigen's HouseholderQ returns a square matrix...
+              } else { 
+                #stop("Unknown (processed) QRmethod")
+                ## lmwithQ_denseZAL but ZAL still Matrix: 
+                ## only valid case would be when ZAL was Identity. (in which cas matrix::qr may be useful *if the matrix is large*)
+                wAugX <- as.matrix(wAugX) ## because both TT and wAugX kept being Matrix              }
+              }
+            } ## end all Matrix cases 
+            ## matrix case not exclusive to Matrix case because of the latest subcase above 
+            if (is.matrix(wAugX)) { 
+              ## wAugX is matrix not Matrix (lmwithQ_denseZAL), with useEigen
+              betaVQ <- lmwithQ(wAugX,wAugz) 
+              betaV <- betaVQ$coef
+              levQ <- betaVQ$Q[,seq_len(ncol(wAugX))] ## Eigen's HouseholderQ returns a square matrix...
+            }
           }
           #if (any(is.nan(betaV))) browser()
-          levQ <- betaVQ$Q[,seq_len(ncol(wAugX))] ## Eigen's HouseholderQ returns a square matrix...
-        } else {
+        } else { ## basic IRLS without use eigen (lmwithQ_denseZAL)
+          SQR <- QRwrap(wAugX)
           betaV <- solveWrap.vector(SQR, wAugz,stop.on.error=stop.on.error) ## qr.coef(SQR, wAugz) ## vector
         }
         if (class(betaV)=="try-error") betaV <- ginv(wAugX)%*% wAugz ## occurred with large lambda either as 'init.HLfit', or by the iterative algo
@@ -298,7 +327,7 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
     if (!is.null(checkv_h)) v_h <- checkv_h
     # print(paste(innerj," ",paste(beta_eta,collapse=" ")),quote=F)
     ####### new values of everything, only tentative if LevenbergM  
-    eta <- off + X.pv %*% beta_eta + ZAL %*% v_h ## updated at each inner iteration
+    eta <- as.matrix(off + X.pv %*% beta_eta + ZAL %id*id% v_h) ## updated at each inner iteration
     ## update functions u_h,v_h
     keep_wranefblob <- wranefblob ####################### 
     wranefblob <- updateW_ranefS(cum_n_u_h,rand.families,lambda_est,u_h,v_h)
@@ -317,21 +346,19 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
       keep_d2hdv2 <- d2hdv2
     }
     if (pforpv>0) {
-      Sig <- ZWZt(ZAL,1/w.ranef) # + diag(1/w.resid) ## ZAL %*% diag(1/w.ranef) %*% t(ZAL) + diag(1/w.resid) ## update v_h -> blob$GLMweights -> w.resid -> Sig -> next beta estimate
-      diag(Sig) <- diag(Sig) + 1/w.resid ## ZAL %*% diag(1/w.ranef) %*% t(ZAL) + diag(1/w.resid) ## update v_h -> blob$GLMweights -> w.resid -> Sig -> next beta estimate
+      Sig <- Sigwrapper(ZAL,1/w.ranef,1/w.resid) ## update v_h -> blob$GLMweights -> w.resid -> Sig -> next beta estimate
     }
-    d2hdv2 <- calcDhDv2(ZAL,w.resid,w.ranef) ##  - t(ZAL) %*% diag(w.resid) %*% ZAL - diag(w.ranef)
-    qr.d2hdv2 <- NULL
+    d2hdv2 <- calcDhDv2(ZAL,w.resid,w.ranef) ## update d2hdv2= - t(ZAL) %*% diag(w.resid) %*% ZAL - diag(w.ranef)
     ######### for LevenbergM, we check there was a progress, and restore everything otherwise
     if (LevenbergM) {      
-      if (HL[1]==0) {
+      if (HL[1]==0L) {
         currentlik <- calc.p_v(mu=mu,u_h=u_h,dvdu=dvdu,lambda_est=lambda_est,phi_est=phi_est,
                                d2hdv2=d2hdv2,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,
                                processed=processed,family=family,prior.weights=prior.weights,only.h=TRUE)$hlik ##  use h in PQL/L (> v1.0)  
       } else currentlik <- calc.p_v(mu=mu,u_h=u_h,dvdu=dvdu,lambda_est=lambda_est,phi_est=phi_est,
                                     d2hdv2=d2hdv2,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,
                                     processed=processed,family=family,prior.weights=prior.weights)$p_v
-      if (innerj==1) { ## not a good idea to use the p_v computed for all u_h initially set to zero
+      if (innerj==1L) { ## not a good idea to use the p_v computed for all u_h initially set to zero
         gainratio <- 1
       } else {
         if (currentlik==-Inf) { ## obs in binary probit with extreme eta... 
@@ -340,11 +367,11 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
           gainratio <- 2*(currentlik-oldlik)/denomGainratio ## cf computation in MadsenNT04 below 3.14, but generalized for D' != I ; numerator is reversed for maximization
         }
       }
-      if (class(try(if (gainratio<0) {}))=="try-error") { ## nnote that NaN *is* numeric!
-        mess <- pastefrom("'try(if (gainratio<0)...) failed.",prefix="(!) From ") 
-        stop(mess)
-      }
-      
+      ## amusing debugging code 
+      #if (class(try(if (gainratio<0) {}))=="try-error") { ## nnote that NaN *is* numeric!
+      #  mess <- pastefrom("'try(if (gainratio<0)...) failed.",prefix="(!) From ") 
+      #  stop(mess)
+      #}
       if (gainratio<0) { ## restore everything
         currentlik <- oldlik ## important to restore this for the next test !
         betaV <- old_betaV
@@ -356,14 +383,13 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
         u_h <- u_h_from_v_h(v_h,rand.families=rand.families,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam) 
         checkv_h <- attr(u_h,"v_h") ## verif is v_h was corrected 
         if (!is.null(checkv_h)) v_h <- checkv_h
-        eta <- off + X.pv %*% beta_eta + ZAL %*% v_h ## updated at each inner iteration
+        eta <- as.matrix(off + X.pv %*% beta_eta + ZAL %id*id% v_h) ## updated at each inner iteration
         wranefblob <- keep_wranefblob ########################## restoration
         w.ranef <- wranefblob$w.ranef ; dlogWran_dv_h <- wranefblob$dlogWran_dv_h ; dvdu <- wranefblob$dvdu
         muetablob <- keep_muetablob ######################################### restoration
         mu <- muetablob$mu ; dmudeta <- muetablob$dmudeta ; w.resid <-as.vector(muetablob$GLMweights /phi_est) 
         if (pforpv>0) Sig <- keep_Sig ###### more restoration
         d2hdv2 <- keep_d2hdv2 ###### more restoration
-        #            qr.d2hdv2 <- keep_qr.d2hdv2 ###### more restoration
         damping <- damping*dampingfactor
         dampingfactor <- dampingfactor*2 
       } else { ## cf Madsen-Nielsen-Tingleff again, and as in levmar library by Lourakis
@@ -386,12 +412,12 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,qr.d2hdv2,w.resid,b
     } 
     if (maxit.mean>1) {
       ## the convergence on v_h^2 must be relative to lambda; this raises questions about the lowest meaningful lambda values.
-      relvariation <- dbetaV*(c(rep(1,pforpv),w.ranef)) ## 21/01/2013 
+      relvariation <- dbetaV*(c(rep(1,pforpv),w.ranef)) 
       if (mean(abs(relvariation)) < conv.threshold) break; ## FR->FR mean(abs) is not standard ?  
     }
   } ## end for (innerj in 1:maxit.mean)
   return(list(beta_eta=beta_eta,v_h=v_h,u_h=u_h,eta=eta,wranefblob=wranefblob,
-              w.resid=w.resid,Sig=Sig,d2hdv2=d2hdv2,qr.d2hdv2=qr.d2hdv2,wAugX=wAugX,tXinvS=tXinvS,
+              w.resid=w.resid,Sig=Sig,d2hdv2=d2hdv2,wAugX=wAugX,tXinvS=tXinvS,
               sqrt.ww=sqrt.ww,innerj=innerj,levQ=levQ,SQR=SQR,muetablob=muetablob)
   )
 } ## end auglinmodfit
