@@ -7,9 +7,11 @@ bfit <- corrHLfit(migStatus ~ means+ Matern(1|longitude+latitude),data=blackcap,
                   ranFix=list(lambda=0.5537,phi=1.376e-05,rho=0.0544740,nu=0.6286311))
 
 ## showing add.map
-filled.mapMM(bfit,add.map=TRUE,plot.axes={axis(1);axis(2)},
+if (require(maps)) { ## required for add.map=TRUE 
+  filled.mapMM(bfit,add.map=TRUE,plot.axes={axis(1);axis(2)},
              plot.title=title(main="Inferred migration propensity of blackcaps",
                               xlab="longitude",ylab="latitude"))
+}
 ## filled.mapMM takes a bit longer
 filled.mapMM(bfit,nlevels=30,plot.axes={axis(1);axis(2)},
              plot.title=title(main="Inferred migration propensity of blackcaps",
@@ -21,9 +23,10 @@ lfit <- corrHLfit(cbind(npos,ntot-npos)~elev1+elev2+elev3+elev4+maxNDVI1+seNDVI
                   family=binomial(),ranFix=list(nu=0.5,rho=2.255197,lambda=1.075))   
 
 ## longer computation requiring interpolation of 197 points 
-filled.mapMM(lfit,add.map=TRUE,plot.axes={axis(1);axis(2)},
-             add.points=quote(points(pred[,coordinates],pch=15,cex=0.3)),
+if (require(maps)) { ## required for add.map=TRUE 
+  filled.mapMM(lfit,add.map=TRUE,plot.axes={axis(1);axis(2)},
+             decorations=quote(points(pred[,coordinates],pch=15,cex=0.3)),
              plot.title=title(main="Inferred prevalence, North Cameroon",
                               xlab="longitude",ylab="latitude"))
-
+}
 # test of syntax, no expect_ yet
\ No newline at end of file
